@@ -1,13 +1,39 @@
+/***************************************************************************
+                          ktail.cpp  -  description
+                             -------------------
+    begin                : Sat Sep 4 1999
+    copyright            : (C) 1999 by Rolf Jakob
+    email                : rjakob@duffy1.franken.de
+ ***************************************************************************/
+
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   * 
+ *                                                                         *
+ ***************************************************************************/
+
 #include <unistd.h>
 #include <errno.h>
+#include <stdlib.h>
 #include <sys/stat.h>
 #include <signal.h>
-#include <ktopwidget.h>
-#include <ktablistbox.h>
+#include <iostream.h>
+//#include <ktopwidget.h>
+//#include <ktablistbox.h>
 #include <kapp.h>
-#include <kmsgbox.h>
-#include <kwm.h>
+#include <kglobal.h>
+#include <kcmdlineargs.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <kmenubar.h>
+#include <kstatusbar.h>
+#include <kpopupmenu.h>
+//#include <kwm.h>
 #include <kfiledialog.h>
+#include <qcursor.h>
 #include <qwidget.h>
 #include <qmlined.h>
 
@@ -18,15 +44,15 @@
 #include <qkeycode.h>
 
 #include "ktail.h"
-#include "getcmdline.h"
+//#include "cmdlinedlg_impl.h"
 
-#include "ktail.moc"
 
 #define M_HSTATUS	1
 #define M_HMENU		2
 #define M_HELP		3
 #define M_REMOVETHIS	4
 #define M_EDITFILTER	5
+#define M_SETTINGS	6
 
 #define REGFILE		1
 #define SPECIALFILE	2
@@ -36,16 +62,32 @@
 
 #define ALLOW_DECOR_CHANGE 1
 
-#define VERSIONSTRING	"0.4.3"
+#define VERSIONSTRING	VERSION
 
-Window mywindow;
+//Window mywindow;
+
+rKTabCtl::rKTabCtl(QWidget *parent)
+	: KTabCtl(parent)
+{ }
+
+void rKTabCtl::mousePressEvent(QMouseEvent *e)
+{
+//cerr << "mouse event in tabctl\n";
+	if (e->button()==RightButton) {
+		emit rightclicked();
+//		cerr << "right mouse button\n";
+	} else if (e->button()==MidButton)
+		emit middleclicked();
+	else KTabCtl::mousePressEvent(e);
+}
 
 ktail :: ktail(KApplication *, const char *name)
-	: KTopLevelWidget(name)
+	: KMainWindow(0,name)
 {
-	char *p,*pf,*fpf;
+	char *p,*pf,*fpf,*ac;
 	char dummy[2];
-	bool has_filters,has_filterflags;
+	bool has_filters,has_filterflags,has_actions,has_maxlines;
+	int action,maxline;
 
 	QString tmp;
 	int ii,found;
@@ -53,8 +95,9 @@ ktail :: ktail(KApplication *, const char *name)
 	*dummy=0;
 	filel.clear();
 
-	dnd=0; tabctl=0; reading=FALSE; do_stop=FALSE;
+	tabctl=0; reading=FALSE; do_stop=FALSE;
 	cmdlinedlg=0;
+	optionw=0;
 
 	menubar=new KMenuBar(this);
 	fileMenu=new QPopupMenu;
@@ -63,13 +106,15 @@ ktail :: ktail(KApplication *, const char *name)
 		this,SLOT(addfiledlg()));
 	fileMenu->insertItem(i18n("&Add pipe ..."),
 		this,SLOT(addcmdline()));
-	fileMenu->insertItem(i18n("&Remove file ..."),
-		this,SLOT(removefile()));
+//	idf1=fileMenu->insertItem(i18n("&Remove file ..."),
+//		this,SLOT(removefile()));
 	fileMenu->insertSeparator();
-	fileMenu->insertItem(i18n("&Clear current log window"),
+	idf2=fileMenu->insertItem(i18n("&Clear current log window"),
 		this,SLOT(clearwindow()),CTRL + Key_C);
-	fileMenu->insertItem(i18n("&Refresh current log window"),
+	idf3=fileMenu->insertItem(i18n("&Refresh current log window"),
 		this,SLOT(reloadwindow()),CTRL + Key_R);
+	fileMenu->insertItem(i18n("&Manage log windows"),
+		this,SLOT(removewindow()));
 	fileMenu->insertSeparator();
 	fileMenu->insertItem(i18n("&Quit"),
 		this,SLOT(close()),CTRL + Key_Q);
@@ -77,9 +122,10 @@ ktail :: ktail(KApplication *, const char *name)
 	menubar->insertItem(i18n("&Options"),optionMenu);
 	optionMenu->insertItem(i18n("&Font ..."),
 		this,SLOT(selectfont()));
+	optionMenu->insertItem(i18n("&Settings"),M_SETTINGS);
 	optionMenu->insertItem(i18n("Hide &Menubar"),M_HMENU);
 	optionMenu->insertItem(i18n("Hide &Statusbar"),M_HSTATUS);
-	
+
 	connect(optionMenu,SIGNAL(activated(int)),this,
 		SLOT(menucallback(int)));
 
@@ -95,14 +141,15 @@ ktail :: ktail(KApplication *, const char *name)
 	connect(popmenu,SIGNAL(activated(int)),this,
 		SLOT(menucallback(int)));
 
-	QString at="ktail "+QString(VERSIONSTRING);
+	menubar->insertSeparator();
+	QString at="ktail " VERSION;
 	at+=i18n("\n\nRolf Jakob (rjakob@duffy1.franken.de)\n\nfile monitor for the KDE Desktop Environment");
-	helpMenu=kapp->getHelpMenu(true,at.data());
-	menubar->insertItem(i18n("&Help"),helpMenu);
+	QPopupMenu *help = helpMenu(at, false);
+	menubar->insertItem(i18n("&Help"),help);
 	menubar->show();
-	setMenu(menubar);
+//	setMenu(menubar);
 
-	KConfig *config=KApplication::getKApplication()->getConfig();
+	KConfig *config=kapp->config();
 	config->setGroup("KTail");
 
 	if (!config->hasKey("font")) {
@@ -113,23 +160,27 @@ ktail :: ktail(KApplication *, const char *name)
 
 	ii=0; found=0;
 
-	statusbar=new KStatusBar(this);
-	statusbar->insertItem("000",STA_LINES);
-	statusbar->insertItem(i18n("initialize"),STA_NAME);
-	statusbar->show();
-	setStatusBar(statusbar);
+//	statusbar=new KStatusBar(this);
+	statusBar()->insertItem("0000",STA_LINES);
+	statusBar()->insertItem(i18n("initialize"),STA_NAME);
+//	statusBar->show();
+//	setStatusBar(statusbar);
 
 	showstatusnonactive=config->readBoolEntry("ShowStatusOfNonActiveFile",false);
 
 	active_tab=0;
 
 	w=new mybox(this);
+	connect(w,SIGNAL(rightclicked()),this,SLOT(togglemenubar()));
+	connect(w,SIGNAL(middleclicked()),this,SLOT(togglestatusbar()));
 
 	createtextbox();
 
-	QStrList files,filters,efilters;
+	QStrList files,filters,efilters,actions,maxlines;
 	has_filters=false;
 	has_filterflags=false;
+	has_actions=false;
+	has_maxlines=false;
 
 	if (config->hasKey("files")) {
 		config->readListEntry("files",files);
@@ -141,20 +192,45 @@ ktail :: ktail(KApplication *, const char *name)
 				has_filterflags=true;
 				}
 			}
+		if (config->hasKey("actions")) {
+			config->readListEntry("actions",actions);
+			has_actions=true;
+			}
+		if (config->hasKey("maxlines")) {
+			config->readListEntry("maxlines",maxlines);
+			has_maxlines=true;
+			}
 		p=files.first();
 		pf=dummy;
 		fpf=dummy;
+		action=0;
+		if (has_actions) {
+			ac=actions.first();
+			if (ac) action=atoi(ac);
+			}
+		if (has_maxlines) {
+			ac=maxlines.first();
+			if (ac) maxline=atoi(ac);
+			}
 		if (has_filters) pf=filters.first();
 		if (has_filterflags) fpf=efilters.first();
 		while(p) {
 			if (has_filters) {
-				addfile(p,pf,*fpf=='e');
+				addfile(p,pf,*fpf=='e',action,maxline);
 				pf=filters.next();
 				if (has_filterflags)
 					fpf=efilters.next();
 				}
-			else addfile(p,"",false);
+			else addfile(p,0,false,action,maxline);
 			p=files.next();
+			if (has_actions) {
+				ac=actions.next();
+				if (ac) action=atoi(ac);
+				}
+			if (has_maxlines) {
+				ac=maxlines.next();
+				if (ac) maxline=atoi(ac);
+				}
 			}
 		}
 
@@ -176,11 +252,11 @@ ktail :: ktail(KApplication *, const char *name)
 	connect(tim,SIGNAL(timeout()),this,SLOT(readintextlist()));
 	tim->start(2000);
 
-	setView(w);
+	setCentralWidget(w);
 
 	if (config->hasKey("statusbar"))
 		if (config->readNumEntry("statusbar")==0)
-			statusbar->hide();
+			statusBar()->hide();
 
 	if (config->hasKey("menubar"))
 		if (config->readNumEntry("menubar")==0)
@@ -194,6 +270,7 @@ ktail :: ktail(KApplication *, const char *name)
 	
 	connect(kapp,SIGNAL(saveYourself()),this,SLOT(saveconfig()));
 
+	setpossibleitems();
 }
 
 void ktail :: createtextbox()
@@ -201,7 +278,11 @@ void ktail :: createtextbox()
 int i;
 file_str *p;
 
-	if (!tabctl) tabctl=new KTabCtl(w);
+	if (!tabctl) {
+		tabctl=new rKTabCtl(w);
+		connect(tabctl,SIGNAL(rightclicked()),this,SLOT(togglemenubar()));
+		connect(tabctl,SIGNAL(middleclicked()),this,SLOT(togglestatusbar()));
+		}
 	w->setChild(tabctl);
 	w->resize(w->sizeHint());
 	w->show();
@@ -216,49 +297,56 @@ file_str *p;
 		while(p) {
 			p->textlist=new mytext(w);
 			p->textlist->show();
-			p->textlist->setReadOnly(TRUE);
-			p->textlist->setAutoUpdate(true);
 			connect(p->textlist,SIGNAL(rightclicked()),
 				this,SLOT(dopopup()));
-			connect(p->textlist,SIGNAL(altrightclicked()),
-				this,SLOT(toggledecoration()));
+//			connect(p->textlist,SIGNAL(altrightclicked()),
+//				this,SLOT(toggledecoration()));
 			connect(p->textlist,SIGNAL(middleclicked()),
 				this,SLOT(togglestatusbar()));
+//			connect(p->textlist,SIGNAL(changed(mytext *)),
+//				this,SLOT(actionOnChange(mytext *)));
 			tabctl->addTab(p->textlist,p->caption);
 			p=filel.next();
 			i++;
 			}
+	setpossibleitems();
 	}
 
 	tabchg(i-1);
 
+//	tabctl->repaint();
+//	repaint();
+
+/*
 	dnd=new KDNDDropZone(tabctl,DndURL);
 	connect(dnd,SIGNAL(dropAction(KDNDDropZone *)),
 	this,SLOT(dropslot(KDNDDropZone *)));
+*/
 }
 
-void ktail :: close()
+void ktail::setpossibleitems()
 {
-	closeEvent((QCloseEvent *)0);
-}
 
-void ktail :: closeEvent(QCloseEvent *e)
-{
-//	saveconfig();
-	(void)e;
-	delete this;
-	kapp->quit();
+if (filel.count()>0) {
+	fileMenu->setItemEnabled(idf1,true);
+	fileMenu->setItemEnabled(idf2,true);
+	fileMenu->setItemEnabled(idf3,true);
+	} else {
+	fileMenu->setItemEnabled(idf1,false);
+	fileMenu->setItemEnabled(idf2,false);
+	fileMenu->setItemEnabled(idf3,false);
+	}
 }
 
 void ktail :: saveconfig()
 {
 file_str *help;
-char line[1024];
+char line[1024],num[16];
 
-	KConfig *config=KApplication::getKApplication()->getConfig();
+	KConfig *config=kapp->config();
 	config->setGroup("KTail");
 
-	QStrList files,regexps,flags;
+	QStrList files,regexps,flags,maxlines,actions;
 	help=filel.first();
 	while(help) {
 		if (help->type==IPCPIPE) {
@@ -268,11 +356,17 @@ char line[1024];
 		else files.append(help->filename);
 		regexps.append(help->textlist->getFilter());
 		flags.append(help->textlist->filterIsExcl()?"e":"i");
+		sprintf(num,"%d",help->textlist->actionCode());
+		actions.append(num);
+		sprintf(num,"%d",help->textlist->getMaxLines());
+		maxlines.append(num);
 		help=filel.next();
 		}
 	config->writeEntry("files",files);
 	config->writeEntry("filters",regexps);
 	config->writeEntry("filterflags",flags);
+	config->writeEntry("actions",actions);
+	config->writeEntry("maxlines",maxlines);
 	config->writeEntry("geometry",size());
 	config->writeEntry("ShowStatusOfNonActiveFiles",showstatusnonactive);
 	config->sync();
@@ -309,7 +403,8 @@ void ktail :: clearwindow()
 file_str *help;
 
 	help=filel.at(active_tab);
-	help->textlist->clear();
+	if (help)
+		help->textlist->clear();
 
 }
 
@@ -318,6 +413,7 @@ void ktail :: reloadwindow()
 file_str *help;
 
 	help=filel.at(active_tab);
+	if (!help) return;
 	if (help->type==IPCPIPE && help->pid)
 		::kill(help->pid,SIGINT);
 	if (help->filep>=0) {
@@ -331,17 +427,17 @@ file_str *help;
 
 void ktail :: togglestatusbar()
 {
-	KConfig *config=KApplication::getKApplication()->getConfig();
+	KConfig *config=kapp->config();
 	config->setGroup("KTail");
-	if (statusbar->isVisible()) {
-		statusbar->hide();
+	if (statusBar()->isVisible()) {
+		statusBar()->hide();
 		optionMenu->changeItem(i18n("Show &Statusbar"),
 			M_HSTATUS);
 		popmenu->changeItem(i18n("Show &Statusbar"),
 			M_HSTATUS);
 		config->writeEntry("statusbar",0);
 		} else {
-		statusbar->show();
+		statusBar()->show();
 		optionMenu->changeItem(i18n("Hide &Statusbar"),
 			M_HSTATUS);
 		popmenu->changeItem(i18n("Hide &Statusbar"),
@@ -354,7 +450,7 @@ void ktail :: togglestatusbar()
 
 void ktail :: togglemenubar()
 {
-	KConfig *config=KApplication::getKApplication()->getConfig();
+	KConfig *config=kapp->config();
 	config->setGroup("KTail");
 	if (menubar->isVisible()) {
 		menubar->hide();
@@ -363,10 +459,11 @@ void ktail :: togglemenubar()
 		popmenu->changeItem(i18n("Show &Menubar"),
 			M_HMENU);
 		if (!config->hasKey("menubar"))
-			KMsgBox::message(this,i18n("ktail Usage"),
+			KMessageBox::information(this,
 			i18n("You may \
 reactivate the menu \n\
-bar by a right click into the text window"));
+bar by a right click into the text window"),
+		i18n("ktail Usage"));
 		config->writeEntry("menubar",0);
 		} else {
 		menubar->show();
@@ -380,6 +477,7 @@ bar by a right click into the text window"));
 	config->sync();
 }
 
+/*
 void ktail :: toggledecoration()
 {
 	long wnow=KWM::getDecoration(mywindow);
@@ -393,14 +491,36 @@ void ktail :: toggledecoration()
 		}
 	show();
 }
+*/
 
 void ktail :: dopopup()
 {
 	popmenu->popup(QCursor::pos());
 }
 
+void ktail::removewindow()
+{
+int rc;
+unsigned int i;
+
+myremovedlg *rd=new myremovedlg(0,i18n("manage log windows"),true);
+for (i=0; i<filel.count(); i++) {
+	rd->insertName(filel.at(i)->filename);
+	}
+rc=rd->exec();
+if (rc==QDialog::Accepted) {
+	for (i=0;i<filel.count();i++)
+		if (rd->isSelected(i))
+			removeit(i);
+	}
+delete rd;
+}
+
 void ktail :: menucallback(int no)
 {
+int maxlines;
+int rc;
+
 	switch(no) {
 	case M_HSTATUS:
 		togglestatusbar(); break;
@@ -408,12 +528,30 @@ void ktail :: menucallback(int no)
 		togglemenubar(); break;
 	case M_REMOVETHIS:
 		removeit(active_tab); break;
+	case M_SETTINGS:
+		if (!optionw) {
+			optionw=new mylogoptdlg(0,i18n("ktail options"),true);
+			maxlines=filel.at(active_tab)->textlist->getMaxLines();
+			optionw->setMaxL(maxlines);
+			optionw->setActionCode(filel.at(active_tab)->textlist->actionCode());
+			rc=optionw->exec();
+			if (rc==QDialog::Accepted) {
+				filel.at(active_tab)->textlist->setMaxLines(optionw->getMaxL());
+				filel.at(active_tab)->textlist->setActionCode(optionw->actionCode());
+				}
+			delete optionw;
+			optionw=0;
+//			connect(optionw,SIGNAL(windowClosed(QWidget *,bool)),
+//			this,SLOT(dialogClosedSlot(QWidget *,bool)));
+			}
+		break;
 	case M_EDITFILTER:
 		filel.at(active_tab)->textlist->editFilter();
 		break;
 	}
 }
 
+/*
 void ktail :: dropslot(KDNDDropZone *z)
 {
 char filen[1024];
@@ -427,6 +565,7 @@ int typ,rc;
 			}
 		}
 }
+*/
 
 void ktail :: tabchanged(int i)
 {
@@ -442,6 +581,7 @@ void ktail :: tabchg(int i)
 char line[1024];
 file_str *help;
 
+
 	if ((unsigned)i<filel.count() && i>=0) {
 
 		help=filel.at(i);
@@ -462,15 +602,16 @@ file_str *help;
 			break;
 		}
 
- 		statusbar->changeItem(line,STA_NAME);
-		sprintf(line,"%3d",help->textlist->numLines());
-		statusbar->changeItem(line,STA_LINES);
+ 		statusBar()->changeItem(line,STA_NAME);
+		sprintf(line,"%3d",help->textlist->paragraphs());
+		statusBar()->changeItem(line,STA_LINES);
 		}
+
 }
 
 void ktail :: selectfont()
 {
-	KConfig *config=kapp->getConfig();
+	KConfig *config=kapp->config();
 	config->setGroup("KTail");
 	KFontDialog::getFont(myfont);
 	config->writeEntry("font",myfont);
@@ -480,44 +621,45 @@ void ktail :: selectfont()
 
 void ktail :: addcmdline()
 {
-	if (!cmdlinedlg) {
-
-	cmdlinedlg=new kgetcmdlinedlg(0,
-		i18n("Enter command line to open with a pipe"),FALSE);
-	cmdlinedlg->setlabel(i18n("Command :"));
-	cmdlinedlg->show();
-	connect(cmdlinedlg,SIGNAL(dialog_closed(bool)),
-		this,SLOT(gotcmdline(bool)));
-	}
-}
-
-void ktail :: gotcmdline(bool flag)
-{
+int rc;
 char line[1024],line2[1024];
 
-//	printf("got dialog_closed signal\n");
-	if (cmdlinedlg && flag) {
-//		printf("flag is ok\n");
+	cmdlinedlg=new mycmdlinedlg(0,
+		i18n("Enter command line to open with a pipe"),true);
+	cmdlinedlg->setlabel(i18n("Command :"));
+	rc=cmdlinedlg->exec();
+	if (rc==QDialog::Accepted) {
 		strcpy(line,cmdlinedlg->getvalue());
-//		printf("value is %s\n",line);
 		if (*line!='|') {
 			strcpy(line2+1,line);
 			*line2='|';
 			strcpy(line,line2);
 			}
-//		printf("adding : \"%s\"\n",line);
-		addfile(line,"",false);
+		addfile(line,0,false);
 		}
 	if (cmdlinedlg) delete cmdlinedlg;
 	cmdlinedlg=0;
 }
 
+void ktail::dialogClosedSlot(QWidget *w, bool flag)
+{
+	if (w==optionw) {
+		if (flag) {
+			filel.at(active_tab)->textlist->setMaxLines(optionw->getMaxL());
+			filel.at(active_tab)->textlist->setActionCode(optionw->actionCode());
+			}
+		delete optionw;
+		optionw=0;
+		}
+}
+
 void ktail :: addfiledlg()
 {
 int rc;
 char path[1024],*p;
 
-	KConfig *config=kapp->getConfig();
+	rc=0;
+	KConfig *config=kapp->config();
 	config->setGroup("KTail");
 
 	if (config->hasKey("lastuseddir"))
@@ -528,7 +670,7 @@ char path[1024],*p;
 		"*",0,i18n("ktail: Open...")) );
 
 	if (!addfilename.isEmpty()) {
-	rc=addfile(addfilename.data(),"",false);
+	rc=addfile((char *)(addfilename.data()),0,false);
 	if (rc>=0) {
 		config->setGroup("KTail");
 		strcpy(path,addfilename.data());
@@ -539,7 +681,8 @@ char path[1024],*p;
 	}
 }
 
-int ktail :: addfile(char *pfilename, char *filter, bool efilter)
+
+int ktail :: addfile(char *pfilename, char *filter, bool efilter, int action,int maxlines)
 {
 struct stat statbuffer;
 file_str *help=new file_str;
@@ -582,11 +725,14 @@ char *p;
 		help->textlist=new mytext(w);
 		help->textlist->show();
 		help->textlist->setFilter(filter,efilter);
-		help->textlist->setReadOnly(TRUE);
+		help->textlist->setActionCode(action);
+		if (maxlines>0)
+			help->textlist->setMaxLines(maxlines);
+//		help->textlist->setReadOnly(TRUE);
 		connect(help->textlist,SIGNAL(rightclicked()),
 			this,SLOT(dopopup()));
-		connect(help->textlist,SIGNAL(altrightclicked()),
-			this,SLOT(toggledecoration()));
+//		connect(help->textlist,SIGNAL(altrightclicked()),
+//			this,SLOT(toggledecoration()));
 		connect(help->textlist,SIGNAL(middleclicked()),
 			this,SLOT(togglestatusbar()));
 		tabctl->addTab(help->textlist,help->caption);
@@ -596,39 +742,24 @@ char *p;
 	/* set new tab the active tab */
 		tabchanged(filel.count()-1);
 		setfont(myfont);
+		setpossibleitems();
 		return(1);
 		}
 	return(-1);
 }
 
-void ktail :: removeit(int no,int)
-{
-int i;
-	
-	QString number=kt->text(no,0);
-	QString file=kt->text(no,1);
-		
-	i=atoi(number.data())-1;
-	
-	removeit(i);
-	
-	delete kt;
-}
-
 void ktail :: removeit(int i)
 {
 file_str *help;
 
 	if (i<0 || (unsigned)i>=filel.count()) {
 		printf("trying to remove non-existant filel(%d)\n",i);
-		delete kt;
 		return;
 		}
 
 	help=filel.at(i);
 	if (!help) {
 		printf("filel.at(%d) is NULL !!\n",i);
-		delete kt;
 		return;
 		}
 
@@ -667,7 +798,7 @@ file_str *help;
 		help=filel.next();
 		}
 
-	delete dnd; dnd=0;
+//	delete dnd; dnd=0;
 	w->removeChild();
 	delete tabctl; tabctl=0;
 	createtextbox();
@@ -676,53 +807,9 @@ file_str *help;
 	do_stop=FALSE;
 	tim->start(2000);
 
-	show();
-}
-
-void ktail :: removefile()
-{
-int i;
-char tmp[1024];
-file_str *p;
-
-	kt=new KTabListBox(0,"",2);
-	kt->setSeparator('\t');
-	kt->setColumn(0,i18n("No."),20);
-	kt->setColumn(1,i18n("File"),100);
-	kt->setAutoUpdate(TRUE);
-	p=filel.first(); i=1;
-	while(p) {
-		sprintf(tmp,"%d\t%s",i,p->filename);
-		kt->insertItem(tmp);
-		p=filel.next();
-		i++;
-		}
-	setallcwidth(kt);
-	kt->show();
-	connect(kt,SIGNAL(highlighted(int,int)),this,SLOT(removeit(int,int)));
-}
-
-void ktail :: setallcwidth(KTabListBox *ktab)
-{
-int i;
-
-	for (i=0;i<ktab->numCols();i++)
-		setcwidth(ktab,i);
-}
-
-void ktail :: setcwidth(KTabListBox *ktab, int col)
-{
-int i,wi;
-char tmp[1024];
-
-	wi=0;
-	QFontMetrics mymetric=kapp->generalFont;
-
-	for (i=0;i<ktab->numRows();i++) {
-		strcpy(tmp,ktab->text(i,col).data());
-		wi=max(wi,mymetric.width(tmp)+15);
-		}
-	ktab->setColumnWidth(col,wi);
+	w->update();
+	update();
+	setpossibleitems();
 }
 
 void ktail :: setfont(QFont fo)
@@ -752,37 +839,37 @@ file_str *i;
 
 void ktail :: openregfile(file_str *f)
 {
-char *line=new char[2050];
+char *line=new char[32768];
 long fpos;
 
 	if (f->filename) {
 		sprintf(line,i18n("trying to open %s"),
 			f->filename);
-		statusbar->changeItem(line,STA_NAME);
+		statusBar()->changeItem(line,STA_NAME);
 		f->filep=open(f->filename,O_RDONLY);
 		if (f->filep<0) {
 			if (f->fstatus>=0) {
 			f->fstatus=-1;
 			sprintf(line,i18n("Could not open regular file %s:\n%s"),
 				f->filename,strerror(errno));
-			KMsgBox::message(this,
+			KMessageBox::error(this,
 				i18n("ktail Error"),line);
 				}
 
 			sprintf(line,i18n("%s not opened"),
 				f->filename);
-			statusbar->changeItem(line,STA_NAME);
+			statusBar()->changeItem(line,STA_NAME);
 			} else {
 			fpos=lseek(f->filep,0,SEEK_END);
-			if (fpos<2048) fpos=0;
-			else	fpos-=2048;
+			if (fpos<32768) fpos=0;
+			else	fpos-=32768;
 			fpos=lseek(f->filep,fpos,SEEK_SET);
 			f->filepos=fpos;
 			if (f->fstatus<0) {
 			sprintf(line,i18n("File %s now open"),
 				f->filename);
 			f->fstatus=0;
-			KMsgBox::message(this,
+			KMessageBox::information(this,
 				i18n("ktail Info"),line);
 				}
 			}
@@ -796,26 +883,26 @@ char *line=new char[2050];
 	if (f->filename) {
 		sprintf(line,i18n("trying to open %s"),
 			f->filename);
-		statusbar->changeItem(line,STA_NAME);
+		statusBar()->changeItem(line,STA_NAME);
 		f->filep=open(f->filename,O_RDONLY);
 		if (f->filep<0) {
 			if (f->fstatus>=0) {
 			f->fstatus=-1;
 			sprintf(line,i18n("Could not open special file %s:\n%s"),
 				f->filename,strerror(errno));
-			KMsgBox::message(this,
+			KMessageBox::error(this,
 				i18n("ktail Error"),line);
 				}
 
 			sprintf(line,i18n("%s not opened"),
 				f->filename);
-			statusbar->changeItem(line,STA_NAME);
+			statusBar()->changeItem(line,STA_NAME);
 			} else {
 			if (f->fstatus<0) {
 			sprintf(line,i18n("File %s now open"),
 				f->filename);
 			f->fstatus=0;
-			KMsgBox::message(this,
+			KMessageBox::information(this,
 				i18n("ktail Info"),line);
 				}
 			}
@@ -931,7 +1018,7 @@ struct stat statbuffer;
 */
 		sprintf(sline,i18n("reading %s"),
 		f->filename);
-		statusbar->changeItem(sline,STA_NAME);
+		statusBar()->changeItem(sline,STA_NAME);
 
 		f->textlist->insertLineAtEnd(line);
 		f->textlist->update();
@@ -961,7 +1048,7 @@ struct stat statbuffer;
 	if (rc>0) {
 		sprintf(sline,i18n("reading %s"),
 		f->filename);
-		statusbar->changeItem(sline,STA_NAME);
+		statusBar()->changeItem(sline,STA_NAME);
 	}
 	if (rc<=0) {
 		if (rc<0) perror(f->filename);
@@ -1013,7 +1100,7 @@ char *p;
 		}
 	sprintf(sline,i18n("reading %s"),
 		f->filename);
-	statusbar->changeItem(sline,STA_NAME);
+	statusBar()->changeItem(sline,STA_NAME);
 	f->fstatus=0;
 	*(line+rc)=0;
 	(f->filepos)+=rc;
@@ -1074,34 +1161,27 @@ unsigned t;
 		} else	tabchg(t);
 
 		tabctl->show();
+
+		if (f->textlist) {
+//			cerr << "action : " << f->textlist->actionCode() << endl;
+			switch(f->textlist->actionCode()) {
+			case ACTION_RAISE:
+				raise();
+//				cerr << "raise" << endl;
+				break;
+			case ACTION_DEICONIFY:
+				show();
+				break;
+			case ACTION_DEICONIFYANDRAISE:
+				show();
+				raise();
+				break;
+			case ACTION_BEEP:
+				kapp->beep();
+				break;
+			}
+			}
 		}
 
 }
 
-int main(int argc, char* argv[])
-{
-int rc,ca;
-
-	KApplication *app=new KApplication(argc,argv);
-	ktail *myktail=new ktail(kapp->getKApplication(),0);
-
-	if (app->isRestored()) { // Session management rules
-		if (KTopLevelWidget::canBeRestored(1))
-			myktail->restore(1);
-	}
-
-	app->setMainWidget(myktail);
-	myktail->show();
-
-	mywindow=KWM::activeWindow();
-
-	if (argc>1)
-	for (ca=1;ca<argc;ca++) {
-		myktail->addfile(argv[ca],"",false);
-		}
-
-	rc=app->exec();
-
-	return(rc);
-}
-
